%% this file generates a list of variables and parameters, used by both matlab and Dynare
% outputs a file "dynare_declarations.m" called upon by the mod file

fileID = fopen('dynare_declarations.m','w');
% shortcuts
fp = @(tex) fprintf(fileID, tex);

fp('%% temporary file, written using Declarations.m \n');
fp('%% called by dynare mod file to declare vars/pars/exos \n \n');



%% list of variables/parameters for matlab

AXN = 18; % states + auxillary vars, keep this updated

vars = {'EtV', 'A', 'c', 'cbar', 'y', 'n', 'lam', 'U', 'K', 'BSRH', 'BLRH', 'BSR', 'BLR', 'BLR0', 'BLR0_e'...
    'QSR', 'QLR', 'PsiH', 'dPsiL_1', 'dPsiS_1', 'dPsiL', 'dPsiS', 'ups', 'delpie', 'Ups', 'iSR', 'iT',...
    'iLR', 'iF', 'mc', 'w', 'wH', 'mu', 'u', 'del', 'alp', 'alpU', 'vpiM', 'vpiI', 'vpiU',...
    't', 'I', 'z', 'zI', 'zp', 'zw', 'lamK', 'pie', 'CBS', 'zeta', 'ELB_shifter'};

pars = {'Abar', 'rho', 'bet', 'sig', 'vsig', 'gam', 'psy', 'nbar', 's', 'eta', 'etaW', 'vphi', 'vphiW',...
    'the', 'xi', 'del0', 'del1', 'del2','vr_pie', 'vr_y', 'pibar', 'rhor',...
    'BLRbar', 'vr_s1', 'vr_s2', 'rhob1', 'rhob2', 'rhoz', 'k', 'ome1', 'ome2', 'ome3', 'vthe', 'ELB', 'norc',...
    'aleph1', 'aleph2', 'aleph3', 'sd_BLR', 'h'};

exos = {'e_a', 'e_z', 'e_R', 'e_BLR', 'ELB_shock'};

% variables
fp('var');
for i = 1:length(vars)
    fp(' ');
    fp(vars{i});
end

% close line
fp('; \n \n');

% parameters
fp('parameters');
for i = 1:length(pars)
    fp(' ');
    fp(pars{i});
end

% close line
fp('; \n \n');

% exogenous
fp('varexo');
for i = 1:length(exos)
    fp(' ');
    fp(exos{i});
end

% close line
fp('; \n \n');

% finally, need to set parameter values

fp('%% set parameter values \n');
fp('for i=1:numel(pars) \n');
fp('    set_param_value(pars{i}, eval([''cal_pars.'' pars{i}])); \n');
fp('end \n');

% auxillary parameters for derivatives
axpars = {'UT1', 'IT1', 'DT1'};

% append first order terms
fp('parameters');
for i = 1:length(axpars)
    fp(' ');
    fp(axpars{i});
end

% second-order terms, AXN set at top of file
for i = 1:AXN
    fp(' ');
    fp(['UT2_' num2str(i)]);
    fp(' ');
    fp(['IT2_' num2str(i)]);
    fp(' ');
    fp(['DT2_' num2str(i)]);
end

% close line
fp('; \n \n');

% set values to zero for now, set properly by steadystate file
fp('set_param_value(');
fp(['''UT1']);
fp(''', 0); \n');
    
fp('set_param_value(');
fp(['''IT1']);
fp(''', 1); \n'); % DISR needs to be nonzero

fp('set_param_value(');
fp(['''DT1']);
fp(''', 0); \n');

for i = 1:AXN
    fp('set_param_value(');
    fp(['''UT2_' num2str(i)]);
    fp(''', 0); \n');
    
    fp('set_param_value(');
    fp(['''IT2_' num2str(i)]);
    fp(''', 0); \n');
    
    fp('set_param_value(');
    fp(['''DT2_' num2str(i)]);
    fp(''', 0); \n');
end
