// declares vars/pars/exos, and sets parameters to calibrated values
@#include "dynare_declarations.m"

// model equations, excluding observation equations 
model;

    // This is the unit root process, which currently does not enter anywhere else 
    ELB_shifter = ELB_shifter(-1) + ELB_shock;

    // Covenience variables first
    #q = EtV ^ (1/(1-gam));
    #uc = (1-bet)*cbar^vsig;
    #ul = (1-bet)*vsig*(nbar/(1+psy))*n^(1+psy);  
    #SDF = bet*zeta*q^(gam+vsig-1)*U(+1)^(-gam)*U^(1-vsig)*lam(+1)/lam;
    #vpi = vpiM + vpiI + vpiU;
    #BSQ = QSR*BSRH;
    #BLQ = QLR*BLRH;

    [name='Auxiliary variable needed for Epstein-Zin preferences']
    EtV=U(+1)^(1-gam);

    // Simplified equations for disr/ups/delpie for MWE
    ups = UT1;
    delpie = DT1;

    log(A) = (1-rho)*log(Abar) + rho * log(A(-1)) + e_a;
    log(zeta) = rhoz * log(zeta(-1)) + e_z;

    // Household
    U = norc*(uc-ul+bet*zeta*q^vsig)^(1/vsig);
    cbar = c - h*c(-1);
    
    [name = 'Euler']
    1 = bet * zeta * (cbar/cbar(+1))^(1/sig) * (U(+1)/q)^(gam/sig) * (1+iSR) / (1+pie(+1));
    wH = nbar * n^psy * cbar^(1/sig);
    lam = (1-bet)*(cbar/U)^(vsig-1);
    [name = 'Household Budget']
    BSQ + BLQ = (BSRH(-1)+(1+k*QLR)*BLRH(-1))/(1+pie) + wH*n + vpi - t - c - PsiH;
    PsiH = 0.5*ome1*(vthe*BSQ-BLQ)+0.5*w*(ome2*(BLRH-BLRH(-1))^2 + ome3*(BSRH-BSRH(-1))^2 );

    // production
    y = (u*K(-1))^the * (A*n)^(1-the);

    // union
    [name = 'NKWPC']
    n*(w-etaW*(w-wH)) = (w/w(-1))*alpU + SDF*(w(+1)/w)*alpU(+1);
    alpU = vphiW*y*((1+pie)/(1+pibar))*(((1+pie)/(1+pibar))*(w/w(-1))-1);

    // Marginal Portfolio Costs
    dPsiL_1 = -w(+1) * ome2 * (BLRH(+1)-BLRH) / QLR;
    dPsiS_1 = -w(+1) * ome3 * (BSRH(+1)-BSRH) / QSR;
    dPsiL = -ome1*(vthe*BSQ-BLQ) + w*ome2*(BLRH-BLRH(-1))/QLR;
    dPsiS = vthe*ome1*(vthe*BSQ-BLQ) + w*ome3*(BSRH-BSRH(-1))/QSR;

    // No-arbitrage relationships
    // note we use (1+k*QLR(+1))/QLR and not 1+iLR
    // Dynare thinks iLR is predetermined due to my time indexing
    [name = 'No Arbitrage LR']
    0 = lam(+1) * (((1+k*QLR(+1))/QLR-dPsiL_1)-(1+dPsiL)*(1+iF));
    [name = 'No Arbitrage SR']
    0 = lam(+1) * ((1+iSR-dPsiS_1)-(1+dPsiS)*(1+iF));

    // Bond market
    QSR = 1/(1+iSR);
    BLR = BLR0 + k/(1+pie+0.003859435)*BLR(-1); // adding on trend growth of real GDP
    iLR = (1+k*QLR(+1))/QLR-1;
    Ups = k/(1+pie)*BLR(-1)*ups+delpie*(BSR(-1)+BLR(-1)*(1+k*QLR))/((1+pie)^2);
    //BLR0 = (1-rhob1)*(1-k/(1+pie))*BLR(-1)+rhob2*(BLRbar-k/(1+pie)*BLR(-1))+e_BLR*(steady_state(y)/100)/QLR;
    BLR0 = (1-rhob1)*(1-k)*BLR(-1)+rhob2*(BLRbar-k/(1+pie+0.003859435)*BLR(-1))+BLR0_e*(steady_state(y)/100)/QLR;

    // Bond Market Clearing
    [name = 'Bond Purchase']
    QSR*BSR=-QLR*BLR; 
    BSR = -BSRH;
    BLR = -BLRH;

    // Pricing
    n/K(-1) = (1-the)*mu/(the*w);
    mc  = ((w/(A*(1-the)))^(1-the)) * ((mu/the)^the);

    [name = 'NKPC']
    (1+s)*(1-eta)+eta*mc = vphi * (alp+1/(1+iSR)/(1+pie(+1))*alp(+1)*y(+1)/y);
    alp = ((1+pie)/(1+pibar))*((1+pie)/(1+pibar)-1);


    // Capital Markets
    z   = (xi/2)*I(-1)*(I/I(-1)-1)^2;
    zI  = (xi/2)*(I/I(-1)-1)^2-xi*I/I(-1)*(I/I(-1)-1);
    K = I + (1-del(-1))*K(-1);
    1 = SDF*(lamK(+1)-zI(+1)) - xi*(I/I(-1)-1)/2;
    lamK = mu*u+(1-del)*SDF*lamK(+1);
    [name='Utilisation FOC']
    mu*K = (del1+del2*(u-1)) * (SDF*lamK(+1));
    del = del0 + del1*(u-1) + del2*(u-1)^2;

    // Profits
    zp = y*(vphi/2)*((1+pie)/(1+pibar)-1)^2;
    zw = y*(vphiW/2)*(((1+pie)/(1+pibar))*(w/w(-1))-1)^2;
    vpiM = y*(1+s-mc)-zp;
    vpiI = mu*u*K(-1)-I-z;
    vpiU = (w-wH)*n-zw;

    // Government
    //iT = rhor*iSR(-1) + (1-rhor)*(steady_state(iSR) + vr_pie * log((1+pie)/(1+pibar)) + vr_y * log(y/y(-1))) + e_R;
    iT = rhor*iSR(-1) + (1-rhor)*(steady_state(iSR) + vr_pie * log((1+pie)/(1+pibar)) + vr_y * log(y/steady_state(y))) + e_R;

    // Occbin equations (interest and QE rules)
    [name='interest', bind='ELB']
    iSR = 0.001247663;
    [name='interest', relax='ELB']
    iSR = iT - 1e-06*vr_s1*Ups;
    [name='eQE', bind='ELB']
    BLR0_e = aleph1 + aleph2 * iT + aleph3 * iT^2 + e_BLR*sd_BLR;
    [name='eQE', relax='ELB']
    BLR0_e = e_BLR;

    // lower bound or not? First equation binds at zero
    CBS = (BSR(-1)+BLR(-1)*(1+k*QLR))/(1+pie);
    t +  CBS = s*y;    
end;

// estimation setup outsourced to external files
@#if scenar == "estimate"
    @#include "estimation_prep.m" // observation equations
    @#include "estimation_bayes.m" // priors, Occbin and estimation()
@#endif

// stochastic simulation without Occbin
@#if scenar == "stoch"
    set_dynare_seed('clock')
    shocks;
        var e_a; stderr 0.0056;
        var e_z; stderr 0.01;
        var e_R; stderr 0.001;
        var e_BLR; stderr 0.1;
        %var e_FG4; stderr 0.001;
        %var e_FG8; stderr 0.001;
    end;
    check;
    stoch_simul(periods=3000, order=1, NOCORR, NOMOMENTS, NOFUNCTIONS, irf=0);          
@#endif