% computes the steady state given parameters
% called by PH_steadystate, which uses this repeatedly to calibrate
% parameters

function [ys,params,check] = PH_steadystate_sublevel(ys,exe, M_)

% Here we load the values of the deep parameters in a loop.

for ii = 1:M_.param_nbr                                 
  eval([ M_.param_names{ii} ' = M_.params(' int2str(ii) ');']);         %    Get the value of parameter i.
end                                                        % End of the loop.  
check = 0;
% END OF THE FIRST MODEL INDEPENDENT BLOCK.


%% THIS BLOCK IS MODEL SPECIFIC.
% Here we define the steady state.
A = Abar;
zeta = 1;
zeta2 = 1;
pie = pibar;
iSR = (1+pie) / bet - 1;
iT = iSR;
iLR = iSR;
iF  = iSR;
kappa = (1+s)*(1-eta)/eta;
SDF = bet; 
ytil = 0;
Ktil = 0;

z = 0; % no adjustment costs in equilibrium
zI = 0; % marginal adjustment cost also zer0
zp = 0;
zw = 0;
alp = 0;
alpU = 0;

%% PH variables all zero
PsiH = 0;
dPsiL = 0;
dPsiL_1 = 0;
dPsiS = 0;
dPsiS_1 = 0;

% depreciation vars
u = 1;
del = del0; 

BLR = 0;
BSR = 0;
QSR = 1/(1+iSR);
QLR = 1/(iLR+1-k);
BLRH = -BLR;
BSRH = -BSR;
BLR0 = 0;
BLR0_e = 0;

disr = IT1;
ups = -(1/disr) * UT1;
delpie = (1/disr) * DT1;

Ups = 0;

% numerically solve for I, w and mu given the above
opts = optimoptions('fsolve', 'Display', 'off', 'FunctionTolerance', 1e-10, 'OptimalityTolerance', 1e-10,...
    'MaxFunctionEvaluations', 3e+05, 'MaxIterations', 4e+05, 'StepTolerance', 1e-10);
tosolve = @(guess) solve_steady_core(guess, del, SDF, pie, eta, the, A, vphi,...
    sig, bet, nbar, psy, alp, iSR, s, etaW, h);

X = fsolve(tosolve, [0.04, 0.3 * Abar, Abar], opts);

% given guesses, solve and verify

mu = X(1);
I = X(2);
w = X(3);

% given w we have wH
wH = ((etaW-1)/etaW)*w;

% given mu, we have lamk (not used here) and iL 
lamK = mu/(1-(1-del)*SDF);

% given I, we have K and L
K = I / del;

% given w and also mu and K, we have n and mc
n = K*(1-the)*mu/(the*w);
mc = (w/(A*(1-the)))^(1-the) * (mu/the)^the;

% this gives us output, consumption and real money
y = K^the * (A*n)^(1-the);
c = y - I;

% c gives us cbar
cbar = (1-h)*c;

% now we solve profits

vpiM = y*(1+s-mc-(vphi/2)*((1+pie)/(1+pibar)-1)^2);
vpiI = mu*K-I-z;
vpiU = n*(w-wH);

% government
CBS = 0;
t = s*y - CBS;

% temp vars
uc = (1-bet)*zeta*cbar^vsig;
ul = (1-bet)*vsig*(nbar/(1+psy))*n^(1+psy);

% utility
U = (norc^vsig * (uc-ul) / (1-(norc^vsig)*bet))^(1/vsig);

% given U, we have EtV and lam
EtV= U^(1-gam);
lam = (1-bet)*zeta*(cbar/U)^(vsig-1);

wal = y - c - I - z - zp; % walras law, must be zero

% observation variables
I_obs = 0;
y_obs = 0;
n_obs = 0;
QE_obs = QLR*BLR/y;
i_obs = iSR;
pie_obs = pie;

% ELB shifter
ELB_shifter = 0;


% END OF THE MODEL SPECIFIC BLOCK.

%% DO NOT CHANGE THIS PART.

% Here we define the steady state values of the endogenous variables of
% the model.
%
params=NaN(M_.param_nbr,1);
for iter = 1:M_.param_nbr %update parameters set in the file
  eval([ 'params(' num2str(iter) ') = ' M_.param_names{iter} ';' ])
end

for ii = 1:M_.orig_endo_nbr %auxiliary variables are set automatically
  eval(['ys(' int2str(ii) ') = ' M_.endo_names{ii} ';']);
end
%%
%% END OF THE SECOND MODEL INDEPENDENT BLOCK.