%% Estimation_bayes.m -- Bayesian estimation of model parameters

%% Declare priors and params to be estimated
estimated_params;

    % shock persistence
    rhoz, normal_pdf, 0.8, 0.2;

    % Taylor
    rhor, normal_pdf, 0.8, 0.2;
    vr_pie, normal_pdf, 1.5, 0.2; 
    vr_y, normal_pdf, 0.5, 0.25; 

    % PH
    ome1, normal_pdf, 1e-02, 0.5;
    ome2, normal_pdf, 1e-01, 0.5;

    % Signalling
    vr_s1, normal_pdf, 2e-02, 2e-02;

    % Rotemburg
    vphi, normal_pdf, 75, 20; 
    vphiW, normal_pdf, 75, 20; 

    % Endogenous QE
    aleph1, normal_pdf, 0, 1; % QE during ZLB regime, constant
    aleph2, normal_pdf, 0, 1; % QE during ZLB regime, times iT
    %aleph3, normal_pdf, 0, 1; % QE during ZLB regime, times iT^2


    % shocks
    % stderr e_a,     gamma_pdf, 0.004, 0.002; 
    % stderr e_z,     gamma_pdf, 0.2,   0.01;
    % stderr e_R,     gamma_pdf, 0.0004, 0.0002;
    % stderr e_BLR,     gamma_pdf, 0.1, 0.05;

    % shocks are truncated normal, bounded to be in [0, inf] later
    stderr e_a,     normal_pdf, 0.0056, 0.0056; 
    stderr e_z,     normal_pdf, 0.01,   0.01;
    stderr e_R,     normal_pdf, 0.001, 0.001;
    stderr e_BLR,     normal_pdf, 0.1, 0.1;
    %corr e_z, e_BLR,  uniform_pdf, , , -0.99, 0.99;

    % measurement errors 
    % use 5% of data standard deviation as mean, half that as SD
    stderr y_obs, gamma_pdf, 0.0026, 0.00131;
    stderr pie_obs, gamma_pdf, 0.00022, 0.00011;
    stderr I_obs, gamma_pdf,0.0074,  0.00374;
    stderr n_obs, gamma_pdf, 0.002, 0.001;
    stderr i_obs, gamma_pdf, 0.00054, 0.00027;
    %stderr QE_obs, gamma_pdf, 0.032, 0.016;

    %sd_BLR, normal_pdf, 1, 5; % scale factor on e_BLR during ZLB

end;

% for stochastic simulation if applicable
shocks;
    var e_a; stderr 0.0056;
    var e_z; stderr 0.01;
    var e_R; stderr 0.001;
    var e_BLR; stderr 0.1;
    
    var y_obs; stderr 0.0026;
    var pie_obs; stderr 0.00022;
    var I_obs; stderr 0.0074;
    var n_obs; stderr 0.002;
    var i_obs; stderr 0.00054;
end;

% initial values not always mean
% estimated_params_init;
% 
% % PH
% ome1, 0;
% ome2, 0;
% 
% % Signalling
% vr_s1, 0;
% 
% end;

% bound some parameters
estimated_params_bounds;

    % Persistences
    rhoz, 0.001, 0.999;
    rhor, 0.001, 0.999;

    % Taylor
    vr_pie, 1.1, inf; 
    vr_y, 0, inf; 
    
    % PH
    ome1, 0, inf;
    ome2, 0, inf;

    % Signalling
    vr_s1, 0, inf;

    % Shocks
    stderr e_a, 0, inf; 
    stderr e_z, 0, inf;
    stderr e_R, 0, inf;
    stderr e_BLR, 0, inf;

    % Multipliers
    %sd_BLR, 0.1, inf;


end;



%% some checks if necessary 
%write_latex_dynamic_model; % helpful when dynare reports "error in equation 27" or whatever
 %identification;
 %dynare_sensitivity;
 %model_diagnostics;

% stochastic simulate for T=3000 periods as a check
%check;
%stoch_simul(periods=3000, order=1, NOCORR, NOMOMENTS, NOFUNCTIONS, irf=0);

%% estimation command
%estimated_params_init(use_calibration); end;

% setup Occbin 
occbin_constraints;
    name 'ELB'; bind iSR < 0.001247663; relax iT - 1e-06*vr_s1*Ups > 0.001247663;
end;

occbin_setup(likelihood_max_kalman_iterations = 50, likelihood_check_ahead_periods = 10000, smoother_maxit = 50, smoother_periods = 10000, likelihood_periodic_solution, smoother_periodic_solution);

% run the estimation
estimation(datafile=data_Hamilton, mode_check, diffuse_filter);